"""
Classifies: CHEBI:15489 3-oxo-fatty acyl-CoA
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_3_oxo_fatty_acyl_CoA(smiles: str):
    """
    Determines if a molecule is a 3-oxo-fatty acyl-CoA.
    
    Args:
        smiles (str): SMILES string of the molecule
    
    Returns:
        bool: True if molecule is a 3-oxo-fatty acyl-CoA, False otherwise
        str: Reason for classification
    """
    
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for key substructures
    
    # CoA core structure pattern (including both protonated and deprotonated phosphates)
    coa_pattern = Chem.MolFromSmarts("[#16]CCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O,OH])(=O)OP([O,OH])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O,OH])([O,OH])=O)n1cnc2c(N)ncnc12")
    
    # 3-oxo fatty acyl pattern - thioester with ketone at position 3
    oxo_pattern = Chem.MolFromSmarts("[C,H]CC(=O)CC(=O)S")
    
    # Check for CoA core
    if not mol.HasSubstructMatch(coa_pattern):
        return False, "Missing coenzyme A core structure"
        
    # Check for 3-oxo fatty acyl group
    if not mol.HasSubstructMatch(oxo_pattern):
        return False, "Missing 3-oxo fatty acyl group"

    # Check for thioester linkage between CoA and fatty acid
    thioester = Chem.MolFromSmarts("C(=O)SC")
    if not mol.HasSubstructMatch(thioester):
        return False, "Missing thioester linkage"

    # Find the fatty acid chain length
    matches = mol.GetSubstructMatches(oxo_pattern)
    if matches:
        # Get atoms in fatty acid chain before the 3-oxo group
        chain_atoms = set()
        def trace_chain(atom, visited):
            if atom.GetIdx() in visited:
                return
            visited.add(atom.GetIdx())
            if atom.GetSymbol() == 'C':
                chain_atoms.add(atom.GetIdx())
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetSymbol() in ['C', 'H'] and neighbor.GetIdx() not in matches[0]:
                        trace_chain(neighbor, visited)
                        
        start_idx = matches[0][0] # First carbon in matching pattern
        trace_chain(mol.GetAtomWithIdx(start_idx), set())
        chain_length = len(chain_atoms)
        
        if chain_length < 2:  # Changed minimum chain length requirement
            return False, "Fatty acid chain too short"
            
        return True, f"3-oxo-fatty acyl-CoA with chain length of approximately {chain_length} carbons"
        
    return False, "Invalid 3-oxo-fatty acyl-CoA structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15489',
                          'name': '3-oxo-fatty acyl-CoA',
                          'definition': 'An oxo fatty acyl-CoA that results '
                                        'from the formal condensation of the '
                                        'thiol group of coenzyme A with the '
                                        'carboxy group of any 3-oxo-fatty '
                                        'acid..',
                          'parents': ['CHEBI:61903']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.10619469026548672 is too low.\n'
               'True positives: '
               "[('CCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N1C=NC2=C1N=CN=C2N', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 24 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 26 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 34 "
               "carbons')]\n"
               'False positives: '
               "[('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCC([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CCCCCC(=O)CC(=O)CC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('CCCC\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 26 "
               "carbons'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCCCC(O)=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('CC(C)=CCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 5 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 28 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 28 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('C[C@H](CCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 9 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC([C@H](C(NCCC(NCCSC(=O)CC(CCC/C=C\\\\CCCCCCCC)=O)=O)=O)O)(C)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 20 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 32 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 34 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 34 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 15 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 14 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 24 "
               "carbons'), "
               "('CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('[H]C(=O)CC([H])=C([H])CC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('[H][C@@](C)(CCC(=O)C(C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 24 "
               "carbons'), "
               "('[H][C@@](C)(CCC(=O)C(C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons'), "
               "('C[C@H](CCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 6 "
               "carbons'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCC(O)=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 6 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 15 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 14 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 13 "
               "carbons'), "
               "('CC\\\\C=C/C[C@H]1[C@@H](CCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)n2cnc3c(N)ncnc23)CCC1=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCC([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 6 "
               "carbons'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(C(C(CC\\\\C(=C\\\\CC4=C(C(=C5C(=C4O)C(OC5)=O)C)OC)\\\\C)=O)C)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 14 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 18 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 11 "
               "carbons'), "
               "('[H]C(CC(O)=O)=C([H])CC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 26 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 7 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 18 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 18 "
               "carbons'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(CC(CCC(C(CCC([O-])=O)=O)C)=O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 7 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 18 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 14 "
               "carbons'), "
               "('S(CCNC(=O)CCNC(=O)[C@H](O)C(COP(OP(OC[C@H]1O[C@@H](N2C3=NC=NC(N)=C3N=C2)[C@H](O)[C@@H]1OP(O)(O)=O)(O)=O)(O)=O)(C)C)C(=O)CC(=O)C/C=C/CC(O)=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 26 "
               "carbons'), "
               "('CC(C)C(=C)CC(=O)C(C)C(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 5 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 32 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 17 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 20 "
               "carbons'), "
               "('C[C@H](CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 5 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCCCCCC([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('C[C@H](CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 7 "
               "carbons'), "
               "('C[C@H](CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 20 "
               "carbons'), "
               "('S(CCNC(=O)CCNC(=O)[C@H](O)C(COP(OP(OC[C@H]1O[C@@H](N2C3=NC=NC(N)=C3N=C2)C(O)C1OP(O)(O)=O)(O)=O)(O)=O)(C)C)C(=O)CC(=O)CCCCCCCCC', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 26 "
               "carbons'), "
               "('C[C@H](CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 7 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 14 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('CC\\\\C=C/C[C@H]1[C@@H](CC(=O)CC(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)n2cnc3c(N)ncnc23)CCC1=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 6 "
               "carbons'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(C4=C5CCC([C@]5(CCC4=O)C)=O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 7 "
               "carbons'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCCCCCCCCCC(O)=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 30 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 30 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 15 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('S(C(=O)CC(C/C=C\\\\CCCCCCCC)=O)CCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]1O[C@@H](N2C3=C(C(=NC=N3)N)N=C2)[C@@H]([C@@H]1OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 17 "
               "carbons'), "
               "('CC(C)CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 13 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 28 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 32 "
               "carbons'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)CCCCC([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 16 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 11 "
               "carbons'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 20 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 28 "
               "carbons'), "
               "('[H]C(=O)CC([H])=C([H])CC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 4 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 9 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP([O-])([O-])=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 12 "
               "carbons'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 8 "
               "carbons'), "
               "('C[C@H](CCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 9 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 10 "
               "carbons'), "
               "('C[C@H](CCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 15 "
               "carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 20 "
               "carbons'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 24 "
               "carbons'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)C(C)C(CC[C@]([C@@]4([C@]5(CC[C@@]6([C@]7(CC[C@H](C[C@]7(C[C@H]([C@]6([C@@]5(CC4)[H])[H])O)[H])O)C)[H])C)[H])(C)[H])=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'3-oxo-fatty acyl-CoA with chain length of approximately 22 "
               "carbons')]\n"
               'False negatives: '
               "[('CCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Fatty acid chain too short (less than 4 carbons)')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 120736,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9991724799947038}


"""
Classifies: CHEBI:25185 menaquinones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_menaquinones(smiles: str):
    """
    Determines if a molecule is a menaquinone (any prenylnaphthoquinone having a prenyl or polyprenyl group
    at position 3 and a methyl group at position 2 on the naphthoquinone ring).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a menaquinone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule is a naphthoquinone
    is_naphthoquinone = False
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 10:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetSymbol() == 'C' for atom in atoms) and sum(atom.GetIsAromatic() for atom in atoms) == 6:
                is_naphthoquinone = True
                break

    if not is_naphthoquinone:
        return False, "Not a naphthoquinone"

    # Check for prenyl/polyprenyl group at position 3
    position_3 = None
    for atom in atoms:
        if sum(bond.GetIsAromatic() for bond in atom.GetBonds()) == 2:
            position_3 = atom.GetIdx()
            break

    if position_3 is None:
        return False, "Could not identify position 3"

    atom_3 = mol.GetAtomWithIdx(position_3)
    neighbors_3 = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom_3.GetNeighbors()]
    prenyl_group = False
    for neighbor in neighbors_3:
        if neighbor.GetSymbol() == 'C' and not neighbor.GetIsAromatic():
            prenyl_group = True
            break

    if not prenyl_group:
        return False, "No prenyl/polyprenyl group at position 3"

    # Check for methyl group at position 2
    position_2 = None
    for atom in atoms:
        if sum(bond.GetIsAromatic() for bond in atom.GetBonds()) == 2:
            position_2 = atom.GetIdx()
            break

    if position_2 is None:
        return False, "Could not identify position 2"

    atom_2 = mol.GetAtomWithIdx(position_2)
    neighbors_2 = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom_2.GetNeighbors()]
    methyl_group = False
    for neighbor in neighbors_2:
        if neighbor.GetSymbol() == 'C' and neighbor.GetDegree() == 1:
            methyl_group = True
            break

    if not methyl_group:
        return False, "No methyl group at position 2"

    return True, "Molecule is a menaquinone"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25185',
                          'name': 'menaquinones',
                          'definition': 'Any prenylnaphthoquinone having a '
                                        'prenyl or polyprenyl group at '
                                        'position 3 and a methyl group at '
                                        'position 2 on the naphthoquinone '
                                        'ring.',
                          'parents': ['CHEBI:26254']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183917,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945627942888}


"""
Classifies: CHEBI:27283 very long-chain fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_very_long_chain_fatty_acid(smiles: str):
    """
    Determines if a molecule is a very long-chain fatty acid (chain length > C22).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a very long-chain fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the number of atoms in the molecule
    num_atoms = mol.GetNumAtoms()

    # Get the number of carbon atoms in the molecule
    num_carbon_atoms = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')

    # Filter out non-fatty acids
    if mol.GetNumAtoms() < 23:
        return False, "Chain length is too short (< C23)"

    # Check for the presence of a carboxyl group
    carboxyl_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O':
            neighbors = atom.GetNeighbors()
            if len(neighbors) == 1 and neighbors[0].GetSymbol() == 'C':
                carboxyl_found = True
                break

    if not carboxyl_found:
        return False, "No carboxyl group found"

    # Check if the molecule is a very long-chain fatty acid
    if num_carbon_atoms > 22:
        if num_carbon_atoms > 27:
            return True, "Ultra-long-chain fatty acid (> C27)"
        else:
            return True, "Very long-chain fatty acid (> C22)"

    return False, "Chain length is too short (< C23)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27283',
                          'name': 'very long-chain fatty acid',
                          'definition': 'A fatty acid which has a chain length '
                                        'greater than C22. Very long-chain '
                                        'fatty acids which have a chain length '
                                        'greater than C27 are also known as '
                                        'ultra-long-chain fatty acids.',
                          'parents': ['CHEBI:35366']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 17,
    'num_false_positives': 100,
    'num_true_negatives': 136,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.1452991452991453,
    'recall': 0.68,
    'f1': 0.23943661971830987,
    'accuracy': 0.5862068965517241}


"""
Classifies: CHEBI:18085 glycosaminoglycan
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_glycosaminoglycan(smiles: str):
    """
    Determines if a molecule is a glycosaminoglycan based on presence of aminomonosaccharide residues.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycosaminoglycan, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"

        # Count atoms
        num_atoms = mol.GetNumAtoms()
        if num_atoms < 10:  # Arbitrary minimum size for GAG
            return False, "Molecule too small for a glycosaminoglycan"

        # Check for presence of nitrogen and oxygen
        num_N = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 7)
        num_O = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8)

        if num_N == 0:
            return False, "No nitrogen atoms found - required for aminosaccharides"
        if num_O < 3:
            return False, "Too few oxygen atoms for glycosaminoglycan structure"

        # Look for amide groups (N-C=O)
        amide_pattern = Chem.MolFromSmarts('[NX3][CX3](=[OX1])')
        num_amides = len(mol.GetSubstructMatches(amide_pattern))

        # Look for hydroxyl groups
        hydroxyl_pattern = Chem.MolFromSmarts('[OH]')
        num_hydroxyls = len(mol.GetSubstructMatches(hydroxyl_pattern))

        # Look for amine groups
        amine_pattern = Chem.MolFromSmarts('[NX3;H2,H1;!$(NC=O)]')
        num_amines = len(mol.GetSubstructMatches(amine_pattern))

        # Calculate ratio of O and N to total atoms
        o_n_ratio = (num_O + num_N) / num_atoms

        # Define criteria for classification
        is_gag = (num_amides >= 1 and  # Contains amide bonds
                 num_hydroxyls >= 1 and  # Contains hydroxyl groups
                 (num_amines >= 1 or num_amides >= 2) and  # Contains amine groups or multiple amides
                 o_n_ratio >= 0.2)  # High proportion of O and N atoms

        if is_gag:
            return True, f"Contains {num_amides} amide bonds, {num_hydroxyls} hydroxyl groups, {num_amines} amine groups"
        else:
            return False, "Does not meet structural requirements for glycosaminoglycan"

    except Exception as e:
        return False, f"Error analyzing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18085',
                          'name': 'glycosaminoglycan',
                          'definition': 'Any polysaccharide containing a '
                                        'substantial proportion of '
                                        'aminomonosaccharide residues.',
                          'parents': ['CHEBI:22506']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('O=C1N(O)CCCCCNC(=O)CCC(=O)N(O)CCCCCCNC(CCC(N(CCCCCCNC(CC1)=O)O)=O)=O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1C)C(=O)NC(C=3SC=C(N3)C(=O)NCC(=O)NC(C(C)C)C4=NC(C5=NC(C6=C(C7=NC(C(NC2CC(=O)NC)=O)=CS7)C=CC(=N6)C=8SC=C(N8)C=9OCC(N9)C(=O)N%10C(C(=O)N)CCC%10)=CS5)=CS4)C(C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('ClC(CCCCCCC[C@@H]([C@H]1NC(=O)[C@H]2N(C(=O)[C@@H](N(C(=O)[C@@H](NC(=O)[C@H](NC(=O)[C@H](CCC(=O)N)NC([C@@H](NC([C@@H](NC(\\\\C(\\\\NC([C@@H](NC([C@@H]1O)=O)C(C)C)=O)=C/C)=O)[C@H](O)C)=O)[C@H](O)C)=O)C)[C@H](OC)C)C)CC(=O)N)CCC2)C)CC', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1NCC[C@H](O)[C@@H]2NC(=O)C(C2=O)=C(O)C=C[C@H]3[C@H](CC=C1)[C@H]4C=C[C@H]5[C@H](C(=O)CO)[C@@H](C[C@@H]5[C@H]4C3)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1NC(CC=CC=C(C=CC=CC(CC=C(C=CC=C1)C)O)C)C/C=C/CCC', 'Error "
               'analyzing molecule: Python argument types in\\n    '
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('CC(=O)N[C@@H]1[C@@H](O)[C@H](O[C@@H]2O[C@H](CO)[C@@H](O[C@@H]3O[C@H](CO[C@H]4O[C@H](CO[C@H]5O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]5O)[C@@H](O)[C@H](O[C@H]5O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]5O)[C@@H]4O)[C@@H](O)[C@H](O[C@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]4O)[C@@H]3O)[C@H](O)[C@H]2NC(C)=O)[C@@H](CO)O[C@H]1NC(=O)C[C@H](N)C(O)=O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O1C=2C=C(C=CC(=O)NCCCCNCCCNC(=O)C=CC3=CC=C1C=C3)C=CC2O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)N[C@@H](C=3SC=C(N3)C(=O)N[C@H](C4=NC(C(N[C@H]2[C@H](CC)C)=O)=C(O4)C)C)CC(=O)O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)NC(C(=O)NC(C(=O)NCC(=O)NC(C(NC(C(N(CC(NC2)=O)C)=O)=C)=O)C)CC=3C4=C(OC)C=CC=C4NC3)CC=5C6=C(C=CC=C6)NC5', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1N([C@H](C(=O)N(CC(=O)N([C@H](C(=O)N2[C@H](C(=O)N(C)[C@H](C(N[C@H](C(N([C@H](C(N([C@H](C(N([C@H]1[C@H](CC)C)C)=O)CC(C)C)C)=O)C(C)C)C)=O)C)=O)CC3=CC=C(OC)C=C3)CCC2)CC(C)C)C)C)C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1NCCC[C@H](NC(=O)C)C(=O)OCCC(=C1)C', 'Error analyzing "
               'molecule: Python argument types in\\n    '
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('Cl[C@@H]1[C@H](O)C2[C@@H]3[C@H](C=CC(O)=C4C(=O)N[C@H](C4=O)[C@@H](O)CCNC(C=CC3)=O)CC2[C@@H]5[C@@H]1[C@@H]([C@H](C)C5)C(OC)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1N([C@H](C(=O)NCCCC[C@H](C(N([C@H]1C(C)C)C)=O)NC(=O)/C=C/C2=CC=C(OCC=C=C)C=C2)C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@@H](C(NC(C(N(CC(N[C@@H]2CO)=O)C)=O)=C)=O)C)CC=3C4=C(OC)C=CC=C4NC3)CC=5C6=C(C=CC=C6)NC5', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C/1N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N([C@H](C(N2[C@H](C(N[C@@H]([C@H](C(N[C@H](C(N\\\\C1=C\\\\C)=O)C(C)C)=O)O)[C@H](CCCCCCCCCCC(=O)CCC)C)=O)CCC2)=O)CC(=O)N)C)[C@H](OC)C)CCC(=O)N)[C@H](O)C)[C@H](O)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@@H](C(NC(C(N(CC(N[C@@H]2C)=O)C)=O)=C)=O)C)CC=3C4=C(OC)C=CC=C4NC3)CC=5C6=C(C=CC=C6)NC5', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1NC=2C(O)=C(C(=O)C(C2)=O)CCC=C(C(OC(=O)C(NC(=O)C3CCCCC3)C)C(C)C(CC=CC=CC=CC(C1)OC)O)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1[C@H]2[C@@H]([C@H](O)[C@H]([C@@H](O)[C@H]([C@H](OC(=O)C)[C@@H]([C@@H](OC)C=CO[C@@]3(C(C4=C5C(C=6O[C@@](C1)(C(=O)NC6C=C5O)C)=C(O)C(=C4O3)C)=O)C)C)C)C)C2', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('ClC1=C(O)C=C2CC(=CC=C[C@@H](OC)C(=O)C[C@@H]([C@H](C=C([C@H](CC(NC1=C2)=O)O)C)C)O)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1NC=2C(=O)C(=C(OC)C(C2[C@H](OC)C(=O)C[C@H](O)C)=O)C[C@H](C[C@H](OC)[C@H](O)[C@H](C=C([C@@H]([C@H](C=CC=C1C)OC)OC(=O)N)C)C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)N[C@H](C=3OC(=C(N3)C(=O)NCC=4SC=C(N4)C(=O)N[C@@H]2C(C)C)C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1C2=NC(=C1)C(=O)N[C@H](C=3SC=C(N3)C(=O)N[C@H](C4=NC(C(N[C@H]2[C@H](CC)C)=O)=C(O4)C)C)CC=5N(C=NC5)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1S[C@@]23N4[C@@H]5[C@@H](OC(=O)C=6C=CC(=C(OC=7C=C([C@@H]([C@]1(N(C)C2=O)C4=O)O)C=CC7O)C6)OC)C=COC=C5[C@H]3O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O([C@H]1[C@@H]([C@H](O)[C@@H]([C@@H](O)[C@@H](CO)C=CC=C(C(=O)N=C2C3=NC4(NC3=C5C(=C2O)C(O)=C(C=6O[C@](OC=C[C@H](OC)[C@H]1C)(C(=O)C56)C)C)CCN(CC4)CC(C)C)C)C)C)C(=O)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C/1N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N([C@H](C(N2[C@H](C(N[C@@H]([C@H](C(N[C@H](C(N\\\\C1=C\\\\C)=O)C(C)C)=O)O)[C@H](CCCCCCCCCCC(O)CCC)C)=O)CCC2)=O)CC(=O)N)C)[C@H](OC)C)CCC(=O)N)[C@H](O)C)[C@H](O)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1O[C@@H](C=C[C@@H](CCC(C=2C=3C(C=4[C@@]1(C=CC(=O)NC4C(=O)C3C=C(C)C2O)C)=O)=O)CC)[C@@H](O)C=C(C)C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S1CCN(O)C(=O)CCC(=O)NCCSCCN(O)C(=O)CCC(NCCSCCN(C(CCC(NCC1)=O)=O)O)=O', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('O=C1N[C@@H](CC=CC=C(C=CC=C[C@H]([C@H](C=C(C=CC=C1)C)O)O)C)C/C=C/C=C/C', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)'), "
               "('S(=O)(=O)(NC(=O)[C@]12NC(=O)[C@H]3N(C[C@H](OC(=O)N4CC5=C(C4)C=CC=C5F)C3)C(=O)[C@@H](NC(OC(C)(C)C)=O)CCCCCC=C[C@@H]1C2)C6CC6', "
               "'Error analyzing molecule: Python argument types in\\n    "
               'rdkit.Chem.rdMolDescriptors.CalcNumAtoms(Mol)\\ndid not match '
               "C++ signature:\\n    CalcNumAtoms(RDKit::ROMol mol)')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 14,
    'num_false_positives': 100,
    'num_true_negatives': 309,
    'num_false_negatives': 15,
    'num_negatives': None,
    'precision': 0.12280701754385964,
    'recall': 0.4827586206896552,
    'f1': 0.1958041958041958,
    'accuracy': 0.7374429223744292}


"""
Classifies: CHEBI:26401 purines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_purines(smiles: str):
    """
    Determines if a molecule is a purine or purine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a purine, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # SMARTS patterns for purine and related structures
        patterns = [
            ('purine_core', 'c1ncnc2[nH]cnc12'),
            ('purine_N9', 'c1ncnc2n(*)cnc12'),  # N9-substituted
            ('purine_N7', 'c1ncnc2ncn(*)c12'),  # N7-substituted
            ('imidazopyrimidine', 'c1ncnc2n1ccn2'),
            ('oxidized_purine', 'c1nc(=O)c2[nH]cnc2n1'),  # For xanthine-like structures
            ('aminopurine', 'c1nc(N)nc2[nH]cnc12')  # For adenine-like structures
        ]
        
        for name, smarts in patterns:
            pattern = Chem.MolFromSmarts(smarts)
            if pattern and mol.HasSubstructMatch(pattern):
                # Get the atoms in the matched core
                matches = mol.GetSubstructMatches(pattern)
                if matches:
                    core_atoms = set(matches[0])
                    
                    # Find substituents
                    substituents = set()
                    for atom_idx in core_atoms:
                        atom = mol.GetAtomWithIdx(atom_idx)
                        for neighbor in atom.GetNeighbors():
                            if neighbor.GetIdx() not in core_atoms:
                                # Add substituent type
                                if neighbor.GetSymbol() != 'H':  # Ignore hydrogens
                                    substituent = neighbor.GetSymbol()
                                    if neighbor.GetDegree() > 1:  # If it's part of a larger group
                                        substituent += "-substituted"
                                    substituents.add(substituent)
                    
                    if substituents:
                        return True, f"Substituted purine ({name}) with substituents: {', '.join(substituents)}"
                    else:
                        return True, f"Unsubstituted purine ({name})"
        
        return False, "Does not contain purine or imidazopyrimidine core"
        
    except Exception as e:
        return False, f"Error analyzing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26401',
                          'name': 'purines',
                          'definition': 'A class of imidazopyrimidines that '
                                        'consists of purine and its '
                                        'substituted derivatives.',
                          'parents': ['CHEBI:35875']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 157,
    'num_false_positives': 100,
    'num_true_negatives': 7466,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.6108949416342413,
    'recall': 0.9457831325301205,
    'f1': 0.7423167848699764,
    'accuracy': 0.9859027418520434}


"""
Classifies: CHEBI:25018 L-leucine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_L_leucine_derivative(smiles: str):
    """
    Determines if a molecule is an L-leucine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an L-leucine derivative, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Create uncharger to neutralize molecule
    uncharger = rdMolStandardize.Uncharger()
    mol = uncharger.uncharge(mol)
    
    # L-leucine core SMARTS patterns
    l_leucine_core = '[C@@H](CC(C)C)(N)[C@@H](=O)O'
    l_leucine_modified = '[C@@H](CC(C)C)(N[*,#1])[C@@H](=O)[OH1,O-,OR,NR,N]'
    
    # Check for L-leucine core or modified core
    core_pat = Chem.MolFromSmarts(l_leucine_core)
    modified_pat = Chem.MolFromSmarts(l_leucine_modified)
    
    if core_pat is not None and mol.HasSubstructMatch(core_pat):
        return True, "Contains L-leucine core structure"
        
    if modified_pat is not None and mol.HasSubstructMatch(modified_pat):
        # Additional checks for derivatives
        for atom in mol.GetAtoms():
            if atom.GetSymbol() == 'N':
                # Check for N-substitution
                if atom.GetDegree() > 1:
                    for neighbor in atom.GetNeighbors():
                        if neighbor.GetSymbol() == 'C' and neighbor.GetChiralTag() == Chem.ChiralType.CHI_TETRAHEDRAL_CCW:
                            isobutyl_pat = Chem.MolFromSmarts('CC(C)C')
                            if mol.HasSubstructMatch(isobutyl_pat):
                                return True, "L-leucine derivative with modified amino or carboxyl group"
                                
    # Check specifically for methyl ester
    methyl_ester_pat = Chem.MolFromSmarts('[C@@H](CC(C)C)(N)C(=O)OC')
    if methyl_ester_pat is not None and mol.HasSubstructMatch(methyl_ester_pat):
        return True, "L-leucine methyl ester"
        
    # Check for N-acyl derivatives
    n_acyl_pat = Chem.MolFromSmarts('[C@@H](CC(C)C)(NC(=O)[*,#1])C(=O)[OH1,O-,OR,NR,N]')
    if n_acyl_pat is not None and mol.HasSubstructMatch(n_acyl_pat):
        return True, "N-acyl L-leucine derivative"
        
    # Check for peptide bonds
    peptide_pat = Chem.MolFromSmarts('[C@@H](CC(C)C)(N)C(=O)N[C@@H]')
    if peptide_pat is not None and mol.HasSubstructMatch(peptide_pat):
        return True, "L-leucine-containing peptide"
    
    return False, "Not an L-leucine derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25018',
                          'name': 'L-leucine derivative',
                          'definition': 'A proteinogenic amino acid derivative '
                                        'resulting from reaction of L-leucine '
                                        'at the amino group or the carboxy '
                                        'group, or from the replacement of any '
                                        'hydrogen of L-leucine by a '
                                        'heteroatom.',
                          'parents': ['CHEBI:47003', 'CHEBI:83811']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.019417475728155338 is too low.\n'
               "True positives: [('[C@@H](CC(C)C)(NC(=O)C)C(O)=O', 'L-leucine "
               "derivative')]\n"
               'False positives: '
               "[('C=1C=2C[C@@H]3[C@]4([C@@](C2C=C(C1)OC)(CCCC4)CCN3C)[H]', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H]([C@@H](NC(=O)[C@H](NC(=O)[C@H](NC(=O)CC(O)CCCCCCC)CC(C)C)CC(=O)O)C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@H](C(N[C@@H](C(N[C@H](C(N[C@H](C(N[C@H]1CC(=O)O)=O)[C@H](CC)C)=O)[C@H](CC)C)=O)CO)=O)CC(C)C)CO)CC(C)C)CC(C)C)C', "
               "'L-leucine derivative'), "
               "('P(O[C@H](C(=O)N[C@H]([C@]1(OC(=O)C2=C(C1)C=CC=C2O)[H])CC(C)C)[C@@H](O)[C@@H](N)CC(=O)N)(O)(O)=O', "
               "'L-leucine derivative'), "
               "('COC1=CC=CC=C1C2=CC=C3[C@H]4[C@H]([C@@H]([C@H](N4CC5CC5)CN3C2=O)CO)C(=O)N6CCOCC6', "
               "'L-leucine derivative'), "
               "('CC=CC1=CC=C2[C@H]3[C@@H](CN2C1=O)[C@@H]([C@H](N3CC4=CN=CN=C4)C(=O)NCC5=CC=C(C=C5)F)CO', "
               "'L-leucine derivative'), ('C[C@H](C[C@H](N)C(O)=O)C(O)=O', "
               "'L-leucine derivative'), "
               "('CCNC(=O)[C@@H]1[C@H]([C@@H]2CN3C(=CC=C(C3=O)C4=CC=CC=C4)[C@@H]2N1CC)CO', "
               "'L-leucine derivative'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)O)CCCNC(=N)N)CC(=O)NCCCC[C@@H]2NC(=O)[C@@H](NC([C@H]([C@H](OC(C[C@H]3C(N[C@H](C(N[C@H]1CCC(=O)OC[C@H](NC(=O)[C@H]4C(C(=O)[C@@H](NC2=O)CC5=CC=CC=C5)CCC4)C(=O)N3)=O)CC(C)C)=O)=O)C)NC(=O)[C@@H](NC(=O)[C@H]6N(C(=O)[C@@H](N)C(C)C)CCC6)CC=7C8=C(C=CC=C8)NC7)=O)CC9=CC=C(O)C=C9', "
               "'L-leucine derivative'), "
               "('O=C1[C@@]2(O)[C@H]3O[C@H]3[C@H]([C@@H]1[C@@H](O)[C@H](C2)NC(=O)/C=C/CCCCCCCCC(=O)O)O', "
               "'L-leucine derivative'), "
               "('[Na+].CCCCCOc1ccc(cc1)-c1cc(no1)-c1ccc(cc1)C(=O)N[C@H]1C[C@@H](O)[C@@H](O)NC(=O)[C@@H]2[C@@H](O)[C@@H](C)CN2C(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H]2C[C@@H](O)CN2C(=O)[C@@H](NC1=O)[C@@H](C)O)[C@H](O)[C@@H](O)c1ccc(O)c(OS([O-])(=O)=O)c1)[C@H](O)CC(N)=O', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H]([C@H](NC(=O)CC(C)C)C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@@H](C(N[C@@H](C(N[C@H](C(N[C@H]1C)=O)CC(C)C)=O)CC(C)C)=O)CC(C)C)CC(C)C)C)C', "
               "'L-leucine derivative'), "
               "('ClC1=C(OC)C=C(O)C2=C1[C@](O[C@@H]3O[C@H]([C@H](OC)[C@](C3)(NO)C)C)([C@H]4C[C@@]5(O)[C@H](N(C)C)C(=O)C(=C([C@]5(C(C4=C2O)=O)O)O)C(=O)N)C', "
               "'L-leucine derivative'), "
               "('CC(C)(C)NC(=O)[C@@H]1C[C@@H]2CCCC[C@H]2CN1C[C@H]([C@H](CC3=CC=CC=C3)NC(=O)[C@H](CC(=O)N)NC(=O)C4=NC5=CC=CC=C5C=C4)O', "
               "'L-leucine derivative'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)[C@@H](C(=O)N[C@@H](CCCCN=C(N)N)C(N[C@H]([C@@H](C(N[C@H](CCC(N(C1=C)C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC(=O)C)CC2=CC=CC=C2)C)/C)=O)C)C[C@H](CC)C)C', "
               "'L-leucine derivative'), "
               "('C1(C(=C([C@H]([C@@]2(C[C@@]3(CC4=C(C=C(C(=C4C(C3=C([C@]12O)O)=O)O)NC(C[NH2+]C(C)(C)C)=O)N(C)C)[H])[H])[NH+](C)C)[O-])C(=O)N)=O', "
               "'L-leucine derivative'), "
               "('CC=CC1=CC=C(C=C1)[C@H]2[C@H](N(C23CN(C3)C(=O)COC)CCC(F)(F)F)CO', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H]([C@@H](NC(=O)[C@H](NC(=O)[C@H](NC(=O)C[C@@H](O)CCCCCCC)CC(C)C)CC(=O)O)C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@H](C(N[C@@H](C(N[C@H](C(N[C@H](C(N[C@H]1CCC(=O)O)=O)C(C)C)=O)CC(C)C)=O)CCC(=O)N)=O)CC(C)C)CO)CC(C)C)CC(C)C)C', "
               "'L-leucine derivative'), "
               "('CC=CC1=CC=C2[C@H]3[C@@H](CN2C1=O)[C@@H]([C@H](N3CC4=CC=CC=C4OC)C(=O)N5CCOCC5)CO', "
               "'L-leucine derivative'), "
               "('O=C1[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C(=C[C@@H]3C=C(CCCC(C[C@H]1CC(=O)C)=O)C)C)CC(C)C', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H](C(=O)N([C@H](C(=O)O[C@@H](C(O)N([C@H](C(O[C@@H](C(N([C@H]1C(C)C)C)=O)C(C)C)=O)CC(C)C)C)C(CC)C)C(C)C)C)C(C)C', "
               "'L-leucine derivative'), "
               "('O=C1N([C@H](C(=O)N2[C@H](C(=O)N([C@H](C(=O)NCC(=O)O[C@@H]([C@@H](C(N[C@@H](C(N3[C@@H](C(N[C@H]1CCC(C)C)=O)C[C@@H](C)C3)=O)CC(C)C)=O)N(C(=O)[C@H]4N(C(=O)[C@@H](N(C(=O)C(=O)CC)C)C(C)C)C[C@H](C4)CC)C)C)CC(C)C)C)CCC2)C(C)C)C', "
               "'L-leucine derivative'), "
               "('CC[C@@H](C)[C@@H]1OC(=O)[C@H](C(C)C)N(C)C(=O)[C@H](CC(C)C)NC(=O)[C@H](C(C)C)N(C)C(=O)[C@H](NC(=O)[C@@H]2CCCN2C(=O)[C@H](Cc2ccccc2)N(C)C(=O)[C@H](Cc2ccccc2)NC(=O)C(NC1=O)C(C)C)[C@H](C)C(C)C', "
               "'L-leucine derivative'), "
               "('S(CC(=O)N[C@H](C(=O)O[C@@H]([C@@H]1NC(=O)[C@@H](N(C(=O)[C@@H](NC(=O)C(N(C(=O)[C@H](OC([C@H]([C@H](OC([C@@H](N(C1=O)C)[C@H](OC)C)=O)C)NC(=O)C)=O)CC2=CC=CC=C2)C)=C)C)C)C)C(C)C)[C@H](O)C(C)C)C', "
               "'L-leucine derivative'), "
               "('O=C/1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)CC(=O)N[C@@H](CCCN=C(N)N)C(NC([C@@H](C(N[C@H](CCC(N\\\\C1=C\\\\C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)=O)C[C@H](CC)C)C', "
               "'L-leucine derivative'), "
               "('O1[C@@H]2N([C@H]3C4=C(O)C(=C(OC)C(=C4[C@@H](CO)N5[C@H]3[C@@H]6[C@H]2C[C@H]([C@H]5C#N)N6C)O)C)CC1', "
               "'L-leucine derivative'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)[C@@H](C(=O)N[C@@H](CC2=CC=C(O)C=C2)C(N[C@H]([C@@H](C(N[C@H](CCC(N(C1=C)C)=O)C(=O)O)=O)C)C=CC(=C[C@@H]([C@@H](OC)CC3=CC=CC=C3)C)C)=O)C)CC(C)C)CC(C)C', "
               "'L-leucine derivative'), "
               "('O=C1O[C@H]([C@H](NC(=O)[C@@](O)(C23O[C@@H](C([C@H](O2)C)CC3)C)C)C(=O)N4NCCC[C@@H]4C(=O)N(O)[C@@H](C)C(N5[C@@H](C(N6[C@H](C(N[C@H]1[C@H](O)C)=O)CCCN6)=O)CCCN5)=O)C(C)C', "
               "'L-leucine derivative'), "
               "('CCNC(=O)[C@@H]1[C@H]([C@@H]2CN3C(=CC=C(C3=O)C=CC)[C@H]1N2CC4=CC(=CC=C4)Cl)CO', "
               "'L-leucine derivative'), "
               "('CC=CC1=CC=C2[C@H]3[C@@H](CN2C1=O)[C@@H]([C@H](N3C(=O)C4CC4)C(=O)N5CCC6=CC=CC=C6C5)CO', "
               "'L-leucine derivative'), "
               "('S1SC(N)C(=O)N[C@H](C(=O)N[C@@]([C@@H](CC)C)(C(=O)N[C@H](C(=O)NC(C(=O)N[C@@H](C(=O)N2[C@H](CCC2)C(=O)N[C@H](CC(C)C)C(O)=O)C1)CC(=O)N)CCC(=O)N)[H])CC3=CC=C(O)C=C3', "
               "'L-leucine derivative'), "
               "('COC1=CC=C(C=C1)C2=CC=C3[C@H]4[C@@H](CN3C2=O)[C@@H]([C@H](N4)C(=O)N5CCOCC5)CO', "
               "'L-leucine derivative'), "
               "('C/C=C\\\\1/CN2[C@]3(C[C@@]1(C4([C@@]2(CC5(C6=CC=CC=C6N(C)[C@]35[H])[C@@]4(OC(\\\\C=C\\\\C7=CC(=C(C(=C7)OC)OC)OC)=O)[H])[H])C(=O)OC)[H])[H]', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@]23C(=O)N[C@H]([C@@H]2C(C)=C([C@H]([C@@H]3C=CC[C@H](C)CCCC(C=C1)=O)O)C)CC4=CC=CC=C4', "
               "'L-leucine derivative'), "
               "('O=C1C=C[C@H](O)CCC[C@H](CC=C[C@@H]2[C@@]13C(=O)N[C@H]([C@@H]3[C@H](C)C(=C2)C)CC4=CC=CC=C4)C', "
               "'L-leucine derivative'), "
               "('O=C1N2[C@@]3(C(=O)NC14C(C(C5=[N+]([O-])C6=C(C75[C@@H]4N8C(=O)[C@@]9%10N(CCC9)C(C8%11C(C(C)(C)C=%12[C@]([C@H]7%11)(O)C%13=C(C%14=C(OC(C)(C)C=C%14)C=C%13)[N+]%12[O-])C%10)=O)C=CC%15=C6C=CC(O%15)(C)C)(C)C)C3)CCC2', "
               "'L-leucine derivative'), "
               "('CC[C@H](C)[C@@H](C(=O)N[C@@]1(CCCC=CCCC[C@](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC1=O)CC(C)C)CS)CCCNC(=N)N)(C)C(=O)N[C@@H](CC2=CNC=N2)C(=O)N[C@@H](CC3=CNC=N3)C(=O)N[C@@H](CO)C(=O)N[C@@H]([C@@H](C)O)C(=O)N)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CC(C)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCC(=O)O)NC(=O)[C@H](CC4=CNC5=CC=CC=C54)NC(=O)CCNC(=O)C', "
               "'L-leucine derivative'), "
               "('OC(=O)[C@H](CCCC(=O)NCC(=O)NC1=CC=CC=C1)C[C@H](N)C(O)=O', "
               "'L-leucine derivative'), "
               "('[H][C@@]12C[C@H]3[C@H](CC)[C@@H](O)N1[C@H]1C[C@@]4([C@H](OC(C)=O)C31)c1ccccc1N[C@@]24[H]', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@@H]1NC(=O)[C@H](NC(=O)[C@@H]2CCCN2C(=O)[C@@H](CC(O)=O)NC(=O)[C@@H](Cc2c[nH]c3ccccc23)NC1=O)c1cccs1', "
               "'L-leucine derivative'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(=O)O)CC2=CC=CC=C2)CC(C)C)CC(=O)N)CCCCN)CO)CC(=O)NCC(=O)N[C@H](C(=O)NCC(N[C@H](C(N[C@H](C(N[C@H]1CC=3C4=C(C=CC=C4)NC3)=O)C)=O)CC(=O)N)=O)CC5=CC=C(O)C=C5', "
               "'L-leucine derivative'), "
               "('O1[C@H](O[C@H]2[C@H](O)[C@@H](O)C(O)O[C@@H]2CO)[C@H](O)[C@@H](O)[C@@H]([C@H]1CO)O[C@H]3O[C@@H]([C@@H](N[C@@H]4[C@H](O)[C@@H](O)[C@H](O)[C@]5([C@@H]4O5)CO)[C@@H]([C@H]3O)O)CO', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](NC(=O)[C@@H](N)[C@@H](C)O)C(O)=O', 'L-leucine "
               "derivative'), "
               "('S([C@H]1C(=O)[C@@]23C(=O)N[C@H]([C@@H]2C(C)=C([C@H]([C@@H]3C=C(C)CCCC(C1)=O)O)C)CC(C)C)C', "
               "'L-leucine derivative'), "
               "('O=C1C2=C[C@H]3[C@@H]([C@@H](O)[C@](O)(C(=O)C)[C@H](C3)C)[C@H]([C@]24C(=O)N[C@H]([C@@H]4C(=C1C)C)CC5=CC=CC=C5)OC(=O)C', "
               "'L-leucine derivative'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)[C@@H](C(=O)N[C@@H](C(C)C)C(N[C@H]([C@@H](C(N[C@H](CCC(N(C1=C)C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)=O)C)CC(C)C)C', "
               "'L-leucine derivative'), "
               "('CC1=C2N3[C@H]([C@H](CC(N)=O)[C@@]2(C)CCC([O-])=O)[C@]2(C)[N+]4=C([C@@H](CCC(N)=O)[C@]2(C)CC(N)=O)C(C)=C2[N+]5=C(C=C6[N+](=C1[C@@H](CCC(N)=O)C6(C)C)[Co--]345C[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc3c(N)ncnc13)[C@@H](CCC(N)=O)[C@]2(C)CC(N)=O', "
               "'L-leucine derivative'), "
               "('CC[C@H](C)[C@H](N(C)C)C(=O)N[C@H]1[C@@H](Oc2ccc(cc2)\\\\C=C/NC(=O)[C@H](CC(C)C)NC1=O)C(C)C', "
               "'L-leucine derivative'), "
               "('ClC(Cl)CCCCCC[C@@H](N)CC(=O)N[C@H](C(=O)N[C@H](C(=O)N([C@H](C(=O)N1[C@H](C(=O)O)CCC1)CC2=CC=C(O)C=C2)C)CC(C)C)CC3=CC=C(O)C=C3', "
               "'L-leucine derivative'), "
               "('COC([C@@H]1C[C@@]23CCCN4CCC5(C6=CC=CC=C6N(C)[C@]15CC2)[C@]34[H])=O', "
               "'L-leucine derivative'), "
               "('O=C1OC([C@@H](NC(=O)[C@H](NC(=O)[C@@H](NC(=O)C[C@@H](O)CCCCCCCCC)CC(C)C)CCC(=O)O)C(=O)N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(N[C@H](C(N[C@@H](C(N[C@H]1C(CC)C)=O)CO)=O)CC(C)C)=O)CO)CC(C)C)C(C)C)C', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](NC(=O)[C@H](CC1=CC=CC=C1)NC(=O)CNC(=O)CNC(=O)[C@@H](N)CC1=CC=C(O)C=C1)C(O)=O', "
               "'L-leucine derivative'), "
               "('O=C1N[C@@H](C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@H](CO)C(N[C@@H](C(N[C@H](C(NC(CC(O[C@@H]([C@@H]1NC(=O)C(NC(=O)[C@H](NC(=O)CC(O)CCCCCCC)CC(C)C)CC(=O)O)C)=O)C(=O)O)=O)[C@H](CC)C)=O)CC(C)C)=O)CC(C)C)CO)CC(C)C)CC(C)C', "
               "'L-leucine derivative'), "
               "('O=C/1NCC(=O)N[C@H](C(=O)N[C@@H](C(=O)O)CC(=O)N[C@@H](CCCN=C(N)N)C(N[C@H]([C@@H](C(N[C@H](CCC(N\\\\C1=C/C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC(=O)C)CC2=CC=CC=C2)C)/C)=O)CC(C)C', "
               "'L-leucine derivative'), "
               "('CC(C)NC(=O)N1CC2(C1)[C@@H]([C@H](N2CCC(F)(F)F)CO)C3=CC=CC=C3', "
               "'L-leucine derivative'), "
               "('C1[C@@H]2[C@H]([C@@H]([C@@H](N2C(=O)C3=NC=CN=C3)C4=CC=CC(=O)N41)C(=O)NCC5=CC(=CC(=C5)F)F)CO', "
               "'L-leucine derivative'), "
               "('C([C@H](CC1=CC=CC=C1)NC(=O)[C@H]2N(CCC2)C([C@H](CCSC)NC(=O)[C@H]3N(CCC3)C(=O)CNC([C@H](CCCC[NH3+])NC([C@H](CC4=CN=CN4)NC([C@H](CO)NC([C@H](CC(C)C)NC([C@H](CCCNC(=[NH2+])N)NC(=O)[C@H]5N(CCC5)C([C@H](CCCNC(=[NH2+])N)NC([C@H]6NC(=O)CC6)=O)=O)=O)=O)=O)=O)=O)=O)(=O)[O-]', "
               "'L-leucine derivative'), "
               "('CCCCCC[C@H](C)C[C@H](C)[C@H]1OC(=O)[C@H](C)NC(=O)[C@H](C)NC(=O)[C@H](CC(C)C)NC(=O)[C@@H](NC(=O)CNC(=O)[C@@H]1C)C(C)C', "
               "'L-leucine derivative'), "
               "('O=C(NC[C@@H]1N2C[C@@H]([C@](C1)(CC2)[H])CC)C=3OC=CC3', "
               "'L-leucine derivative'), "
               "('O=C1[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C(=C[C@@H]3CC(C)=CC[C@H](C1)O)C)CC(C)C', "
               "'L-leucine derivative'), "
               "('C12=CC=CC=C1NC3=C2[C@]4([C@H]([C@](CC[C@@]4(C3(C)C)[H])(C)C=C)[N+]#[C-])[H]', "
               "'L-leucine derivative'), "
               "('ClC=1C(=O)C(Cl)=CC2(C1)OC(O)[C@H](C2)NC(=O)/C=C/C(=C/C(CCCCCC)C)/C', "
               "'L-leucine derivative'), "
               "('O=C(N[C@H](C(=O)OC[C@@H](NC(=O)C1=CC=CC=C1)CC2=CC=CC=C2)CC(C)C)C3=CC=CC=C3', "
               "'L-leucine derivative'), "
               "('CN1[C@H]2[C@H](CN3C2=CC=C(C3=O)C4=CC=CC=C4F)[C@H]([C@@H]1C(=O)NCC(F)(F)F)CO', "
               "'L-leucine derivative'), "
               "('O=C1[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C(=C[C@@H]3C=C(CC[C@H]([C@H]([C@@H](C1)OC)O)O)C)C)CC(C)C', "
               "'L-leucine derivative'), "
               "('ClC(Cl)CCCCCC[C@@H](N)[C@@H](O)C(=O)N[C@H](C(=O)N([C@H](C(=O)N1[C@H](C(=O)N[C@H](C(=O)O)CC2=CC=C(O)C=C2)CCC1)CC(C)C)C)[C@H](CC)C', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](NC(=O)[C@H](C)N)C(=O)N[C@@H](C)C(=O)N1CCC[C@H]1C(O)=O', "
               "'L-leucine derivative'), "
               "('C1C=2C=3C(NC2[C@]4(N5[C@@]1([C@]([C@](C4)(/C(/C5)=C(\\\\C)/[H])[H])(CO)C(OC)=O)[H])[H])=CC=CC3', "
               "'L-leucine derivative'), "
               "('O=C/1N[C@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)O[C@@H](CC(N[C@H](C(N[C@H](C(N[C@H](C(N\\\\C1=C\\\\C)=O)CCN)=O)CC(C)C)=O)CN)=O)CCCCCCCCC)CC(=O)O)C(O)(C)C)CC2=CC=CC=C2)CN)[C@H](O)C', "
               "'L-leucine derivative'), ('*C([C@H](C1C(C)C1)N*)=O', "
               "'L-leucine derivative'), "
               "('CCN1[C@@H]2CN3C(=CC=C(C3=O)C=CC)[C@H]1[C@H]([C@@H]2CO)C(=O)N4CCC5=CC=CC=C5C4', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C(=C[C@@H]3C=CC[C@@H](C([C@@H](C=C1)C)O)C)C)CC4=CC=C(O)C=C4', "
               "'L-leucine derivative'), "
               "('O=C1C=CCCC(=C[C@H](CC=C[C@@H]2[C@@]13C(=O)N[C@H]([C@@H]3[C@H](CC)C(=C2)C)[C@@H](C=4C5=C(C=CC=C5)NC4)C)C)C', "
               "'L-leucine derivative'), "
               "('CCS(=O)(=O)N1CC2(C1)[C@H]([C@H](N2CC3=CC=C(C=C3)F)CO)C4=CC=CC=C4', "
               "'L-leucine derivative'), "
               "('CCCNC(=O)[C@@H]1[C@H]([C@@H]2CN3C(=CC=C(C3=O)C=CC)[C@H]1N2C(=O)NC4=C(C=CC(=C4)F)F)CO', "
               "'L-leucine derivative'), "
               "('O=C1C(=C[C@H](CC=C[C@@H]2C3(C=4C1=CNC4)C(=O)N[C@H]([C@@H]3[C@H](C)C([C@H]2O)=C)CC=5C6=C(C=CC=C6)NC5)C)C', "
               "'L-leucine derivative'), "
               "('O=C(N[C@H](C(=O)N[C@H](CN)CC(C)C)[C@H](CC)C)C(NC(=O)[C@@H](NC(=O)C(NC(=O)[C@@H](NC(=O)[C@H]1N(C(=O)[C@H]2N(C(=O)CCCCCCCCC)C[C@@H](C2)O)C[C@@H](C1)O)C(C)C)(C)C)CCC(=O)N)(C)C', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H]([C@H](NC(=O)[C@@H](NC=O)[C@@H](CC)C)C(=O)N[C@@H](C(=O)N[C@H](C(C)C)C(N2[C@H](C(N[C@H](C(N([C@H]1C(C)C)C)=O)CC(C)C)=O)C[C@H](O)CN2)=O)CC3=CC=C(OC)C=C3)C', "
               "'L-leucine derivative'), "
               "('[H][C@@]1(Cc2ccc(O)cc2)NC(=O)[C@H](CCC(N)=O)NC(=O)[C@H](CC(N)=O)NC(=O)[C@@H](Cc2ccccc2)NC(=O)[C@H](Cc2c[nH]c3ccccc23)NC(=O)[C@]2([H])CCCN2C(=O)[C@@H](Cc2ccccc2)NC(=O)[C@H](CC(C)C)NC(=O)[C@H](CCCN)NC(=O)[C@@H](NC1=O)C(C)C', "
               "'L-leucine derivative'), "
               "('[C@@H]1(C(=O)N[C@H](C(N[C@@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(=O)N[C@H](C(=O)N1)CCC(=O)O)C(C)O)=O)CCC(O)=O)=O)CO)=O)CC(C)C)=O)CC(C)C)=O)CCCCN)C(O)C', "
               "'L-leucine derivative'), "
               "('CC(C)CCCCC(=O)N[C@@H](CCN)C(=O)N[C@@H]([C@@H](C)O)C(=O)N[C@@H](CCN)C(=O)N[C@H]1CCNC(=O)[C@@H](NC(=O)[C@H](CCN)NC(=O)[C@H](CCN)NC(=O)[C@H](CC(C)C)NC(=O)[C@@H](CC(C)C)NC(=O)[C@H](CCN)NC1=O)[C@@H](C)O', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](NC(=O)[C@H](CO)NC(=O)[C@@H]1CCCN2N1C(=O)[C@H](CCC2=O)NC(=O)[C@H](C)NC(=O)[C@H](CCCNC(N)=N)NC(=O)[C@H](CC1CCCCC1)NC(C)=O)C(N)=O', "
               "'L-leucine derivative'), "
               "('COCCOC(=O)N1C2=C(C=C(C=C2)C#CCC(C(=O)OC)C(=O)OC)[C@@]3(C1=O)[C@H]([C@@H]4C(=O)O[C@H]([C@H](N4[C@H]3C5=CC=CC=C5OCCO)C6=CC=CC=C6)C7=CC=CC=C7)C(=O)N8CCN(CC8)C9=NC=CC=N9', "
               "'L-leucine derivative'), "
               "('O=C(N1[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](CO)CC(C)C)CCC(=O)N)C)CC(C)C)C[C@H](C1)O)C(NC(=O)CCCCCCCCCCCCC)(C)C', "
               "'L-leucine derivative'), "
               "('O=C(N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)O)CC(C)C)[C@H](CC)C)C)[C@](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)CNC(=O)CCCCCCC)C)CC(C)C)(CC)C', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C([C@H]([C@@H]3C=CC[C@H](C)CCC[C@H](C=C1)O)O)=C)CC4=C(O)C=CC=C4', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](N)C(=O)N[C@@H](CC(C)C)C(O)=O', 'L-leucine "
               "derivative'), "
               "('C1CCC(C1)C(=O)N2CC3(C2)[C@@H]([C@H](N3CC4CC4)CO)C5=CC=CC=C5', "
               "'L-leucine derivative'), "
               "('CC(C)C[C@H](NC(=O)[C@@H](N)CC(O)=O)C(=O)N[C@@H]([C@@H](C)O)C(=O)N[C@@H](CC(O)=O)C(O)=O', "
               "'L-leucine derivative'), "
               "('O=C1O[C@@H]([C@@H](NC(=O)[C@H](NC(=O)[C@@H](NC(=O)C[C@H](O)CCCCCCCCC)CC(C)C)CCC(=O)O)C(=O)N[C@@H](C(=O)N[C@@H](C(=O)N[C@@H](C(N[C@H](C(N[C@@H](C(N[C@H]1[C@H](CC)C)=O)CO)=O)CC(C)C)=O)CO)CC(C)C)[C@H](CC)C)C', "
               "'L-leucine derivative'), "
               "('O=C1O[C@H]([C@H](CCCCCC)C)CC(=O)N[C@@H](C(C)C)C(N[C@H](C(N[C@@H]1CC(C)C)=O)C)=O', "
               "'L-leucine derivative'), "
               "('C(=O)([C@@H](N)CCC(=O)O)N[C@H](C(=O)N[C@H](C(=O)O)CC(C)C)CC(=O)O', "
               "'L-leucine derivative'), "
               "('ClCCCCCCC[C@@H](N)[C@@H](O)C(=O)N[C@H](C(=O)N([C@H](C(=O)N1[C@H](C(=O)N[C@H](C(=O)O)CC2=CC=C(O)C=C2)CCC1)CC(C)C)C)[C@H](CC)C', "
               "'L-leucine derivative'), "
               "('C[C@H]1[C@H]2[C@H](Cc3c[nH]c4ccccc34)NC(=O)[C@@]22[C@@H](\\\\C=C\\\\C[C@H](C)\\\\C=C(C)\\\\[C@@H](OC(C)=O)C(=O)\\\\C=C\\\\C2=O)C2O[C@]12C', "
               "'L-leucine derivative'), "
               "('O=C/1N[C@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)O[C@@H](CC(N[C@H](C(N[C@H](C(N[C@H](C(N\\\\C1=C\\\\C)=O)CCN)=O)CC(C)C)=O)CN)=O)CCCCCC(C)C)CC(=O)O)C(C)C)CC2=CC=CC=C2)CN)[C@H](O)C', "
               "'L-leucine derivative'), "
               "('CCC(=O)N1CC2(C1)[C@@H]([C@H](N2S(=O)(=O)C)CO)C3=CC=C(C=C3)C4=CC(=CC=C4)C', "
               "'L-leucine derivative'), "
               "('O=C(N[C@H](C(=O)N[C@H](CN)CC(C)C)CC(C)C)C(NC(=O)[C@@H](NC(=O)C(NC(=O)[C@@H](NC(=O)[C@H]1N(C(=O)[C@H]2N(C(=O)CCCCCCCCC)C[C@@H](C2)O)C[C@@H](C1)O)C(C)C)(C)C)CCC(=O)N)(C)C', "
               "'L-leucine derivative'), "
               "('O=C1NC=2C(C(=O)[C@H]3OC3(C)C)=CC=CC2[C@@]14C([C@]5(O)C[C@@H]6CCC[C@H](N6C[C@]5(C4)NC)C)(C)C', "
               "'L-leucine derivative'), "
               "('O=C1N([C@H](C(=O)N([C@H](C(=O)N[C@H](C(=O)N(C)[C@H](C(N([C@@H](C(N[C@@H](C(N[C@@H]1[C@H](CC)C)=O)[C@H](O)C(C)C)=O)C)C)=O)C)CC(C)C)CC(C)C)C)C)C', "
               "'L-leucine derivative'), "
               "('O=C(N1[C@H](C(=O)N[C@H](C(=O)NC(C(=O)N[C@H](C(=O)N[C@H](C(=O)NC(C(=O)NC(C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](CO)CC(C)C)C)CCC(=O)N)(C)C)(C)C)CCC(=O)N)CCC(=O)N)(C)C)CC2=CC=CC=C2)CCC1)C(NC(=O)CCCCCCCCC)(C)C', "
               "'L-leucine derivative'), "
               "('O=C1[C@@H](O)CC(=O)O[C@@]23C(=O)N[C@H]([C@@H]2[C@H](C)C(=C[C@@H]3C=C(CC[C@H]1O)C)C)CC(C)C', "
               "'L-leucine derivative')]\n"
               "False negatives: [('COC(=O)[C@@H](N)CC(C)C', 'Not an L-leucine "
               "derivative')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 5136,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9809087437953418}


"""
Classifies: CHEBI:149453 indenofuran
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_indenofuran(smiles: str):
    """
    Determines if a molecule is an indenofuran (an organic heterotricyclic compound whose skeleton consists of an indene fused to a furan ring).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an indenofuran, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the ring information
    rings = mol.GetRingInfo()

    # Check for exactly three rings
    if len(rings.AtomRings()) != 3:
        return False, "Molecule does not have exactly three rings"

    # Check if the rings are fused
    fused_rings = []
    for ring1_idx, ring1 in enumerate(rings.AtomRings()):
        for ring2_idx, ring2 in enumerate(rings.AtomRings()):
            if ring2_idx > ring1_idx:
                common_atoms = set(ring1) & set(ring2)
                if len(common_atoms) > 1:
                    fused_rings.append((ring1, ring2))

    if len(fused_rings) != 2:
        return False, "Molecule does not have two fused rings"

    # Check if one of the rings is a 5-membered ring (furan)
    has_furan = False
    for ring in rings.AtomRings():
        if len(ring) == 5:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if any(atom.GetSymbol() == 'O' for atom in atoms):
                has_furan = True
                break

    if not has_furan:
        return False, "Molecule does not contain a furan ring"

    # Check if the other ring is an indene
    has_indene = False
    for ring1, ring2 in fused_rings:
        if len(ring1) == 6 and len(ring2) == 6:
            atoms1 = [mol.GetAtomWithIdx(i) for i in ring1]
            atoms2 = [mol.GetAtomWithIdx(i) for i in ring2]
            if all(atom.GetSymbol() == 'C' for atom in atoms1 + atoms2):
                has_indene = True
                break

    if not has_indene:
        return False, "Molecule does not contain an indene ring"

    return True, "Molecule is an indenofuran"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:149453',
                          'name': 'indenofuran',
                          'definition': 'Any organic heterotricyclic compound '
                                        'whose skeleton consists of an indene '
                                        'fused to a furan ring.',
                          'parents': ['CHEBI:26979']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 44053,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9977125515242107}


"""
Classifies: CHEBI:27116 trihydroxyflavone
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_trihydroxyflavone(smiles: str):
    """
    Determines if a molecule is a trihydroxyflavone (any hydroxyflavone carrying three hydroxy groups at unspecified positions).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a trihydroxyflavone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if it is a flavone
    isFlavone = rdMolDescriptors.CalcPAFlavoneDescriptor(mol)
    if not isFlavone:
        return False, "Not a flavone"

    # Count the number of hydroxy groups
    num_hydroxy = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1)

    if num_hydroxy == 3:
        return True, "Trihydroxyflavone"
    elif num_hydroxy < 3:
        return False, "Not enough hydroxy groups"
    else:
        return False, "Too many hydroxy groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27116',
                          'name': 'trihydroxyflavone',
                          'definition': 'Any  hydroxyflavone carrying three '
                                        'hydroxy groups at unspecified '
                                        'positions.',
                          'parents': ['CHEBI:24698']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.rdMolDescriptors' has no attribute "
             "'CalcPAFlavoneDescriptor'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:133135 chromenochromene
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole

def is_chromenochromene(smiles: str):
    """
    Determines if a molecule is a chromenochromene (contains two ortho-fused chromene rings).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a chromenochromene, False otherwise
        str: Reason for classification
    """
    
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Generate the ring information
    rings = mol.GetRingInfo()
    
    # SMARTS pattern for chromene ring
    chromene_pattern = Chem.MolFromSmarts("[#6]1[#6][#6][#6]2[#8][#6][#6][#6][#6]2[#6]1")
    
    # Find all matches of chromene pattern
    matches = mol.GetSubstructMatches(chromene_pattern)
    
    if len(matches) < 2:
        return False, "Does not contain at least two chromene rings"
        
    # Check if any two chromene rings are ortho-fused
    found_ortho_fusion = False
    for i in range(len(matches)):
        for j in range(i+1, len(matches)):
            ring1_atoms = set(matches[i])
            ring2_atoms = set(matches[j])
            
            # Check for shared atoms (fusion)
            shared_atoms = ring1_atoms.intersection(ring2_atoms)
            
            if len(shared_atoms) >= 2:  # At least 2 atoms shared for ortho fusion
                # Get the bonds between shared atoms to confirm ortho fusion
                shared_atoms_list = list(shared_atoms)
                bond_count = 0
                for idx1 in range(len(shared_atoms_list)):
                    for idx2 in range(idx1+1, len(shared_atoms_list)):
                        bond = mol.GetBondBetweenAtoms(shared_atoms_list[idx1], 
                                                      shared_atoms_list[idx2])
                        if bond is not None:
                            bond_count += 1
                            
                if bond_count >= 1:  # Confirms ortho fusion
                    found_ortho_fusion = True
                    break
                    
        if found_ortho_fusion:
            break
            
    if not found_ortho_fusion:
        return False, "Chromene rings are not ortho-fused"
        
    # Count number of substituents
    substituent_count = 0
    for atom in mol.GetAtoms():
        if atom.GetDegree() > 2 and atom.IsInRing():
            for neighbor in atom.GetNeighbors():
                if not neighbor.IsInRing():
                    substituent_count += 1
                    
    if substituent_count > 0:
        return True, f"Chromenochromene with {substituent_count} substituents"
    else:
        return True, "Unsubstituted chromenochromene"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:133135',
                          'name': 'chromenochromene',
                          'definition': 'Any organic heteroolycyclic compound '
                                        'whose skeleton two ortho-fused '
                                        'chromene rings, and their '
                                        'derivatives.',
                          'parents': ['CHEBI:38166']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 17261,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 1.0,
    'f1': 0.07407407407407407,
    'accuracy': 0.994241289951051}


"""
Classifies: CHEBI:25418 morphinane alkaloid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_morphinane_alkaloid(smiles: str):
    """
    Determines if a molecule is a morphinane alkaloid based on the morphinan skeleton.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a morphinane alkaloid, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Key structural features of morphinane alkaloids:
    # 1. Must contain nitrogen (N)
    # 2. Must have 4 or 5 rings total
    # 3. Must have specific ring systems characteristic of morphinan skeleton
    
    # Check for nitrogen
    if not any(atom.GetSymbol() == 'N' for atom in mol.GetAtoms()):
        return False, "No nitrogen atom found"

    # Count rings
    ring_count = rdMolDescriptors.CalcNumRings(mol)
    if ring_count < 4 or ring_count > 5:
        return False, f"Incorrect number of rings ({ring_count}). Expected 4-5 rings."

    # Check for characteristic ring systems using SMARTS patterns
    # Morphinan core structure patterns
    morphinan_patterns = [
        # Basic morphinan skeleton with N-containing ring
        "[#6]1[#6][#6][#7][#6][#6]2[#6][#6][#6][#6][#6]3[#6][#6][#6][#6][#6]123",
        # Alternative pattern with oxygen bridge
        "[#6]1[#6][#6][#7][#6][#6]2[#6][#6]3[#6][#6][#6][#6][#8][#6]2[#6]13"
    ]

    matches_core = False
    for pattern in morphinan_patterns:
        if mol.HasSubstructMatch(Chem.MolFromSmarts(pattern)):
            matches_core = True
            break
            
    if not matches_core:
        return False, "Does not match morphinan core structure"

    # Additional checks for typical substituents
    common_substituents = []
    
    # Check for hydroxyl groups
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[#8H1]")):
        common_substituents.append("hydroxyl")
        
    # Check for methoxy groups
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[#6][#8][#6]")):
        common_substituents.append("methoxy")
        
    # Check for carbonyl groups
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[#6]=O")):
        common_substituents.append("carbonyl")

    substituents_str = ", ".join(common_substituents) if common_substituents else "no common"
    
    return True, f"Matches morphinan core structure with {substituents_str} substituents"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25418',
                          'name': 'morphinane alkaloid',
                          'definition': 'An  isoquinoline alkaloid based on a '
                                        'morphinan skeleton and its '
                                        'substituted derivatives.',
                          'parents': ['CHEBI:24921']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183851,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999564883959992}


"""
Classifies: CHEBI:17264 phosphatidylglycerol phosphate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_phosphatidylglycerol_phosphate(smiles: str):
    """
    Determines if a molecule is a phosphatidylglycerol phosphate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a phosphatidylglycerol phosphate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for required phosphate groups
    phosphate_pattern = Chem.MolFromSmarts('[P](=O)([O,OH])[O,OH]')
    phosphate_matches = mol.GetSubstructMatches(phosphate_pattern)
    if len(phosphate_matches) < 2:
        return False, "Missing required phosphate groups"
        
    # Check for glycerol backbone with ester linkages
    glycerol_pattern = Chem.MolFromSmarts('OCC(O)COP(=O)(O)O')
    if not mol.HasSubstructMatch(glycerol_pattern):
        return False, "Missing glycerol backbone with phosphate"
        
    # Check for fatty acid ester linkages
    ester_pattern = Chem.MolFromSmarts('C(=O)OC')
    ester_matches = mol.GetSubstructMatches(ester_pattern)
    if len(ester_matches) < 2:
        return False, "Missing required fatty acid ester linkages"
        
    # Check for terminal phosphate group
    terminal_phosphate = Chem.MolFromSmarts('COP(=O)(O)O')
    if not mol.HasSubstructMatch(terminal_phosphate):
        return False, "Missing terminal phosphate group"
        
    # Count total oxygens to verify phosphatidylglycerol phosphate structure
    num_o = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8)
    if num_o < 11:  # Minimum oxygens needed for core structure
        return False, "Insufficient oxygen atoms for phosphatidylglycerol phosphate"

    return True, "Contains phosphatidylglycerol phosphate core structure with required functional groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17264',
                          'name': 'phosphatidylglycerol phosphate',
                          'definition': 'A phosphatidylglycerol in which one '
                                        'of the hydroxy groups of the glycerol '
                                        'moiety has been converted to the '
                                        'corresponding dihydrogen phosphate.',
                          'parents': ['CHEBI:17517']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 13962,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9928901528617134}